# -*- coding: utf-8 -*-
'''
This package contains classes related to games' collection storage in .pgn files.
'''


########################################################################
from gamerec.config import *
__author__  = gamerec_author
__status__  = gamerec_status
__version__ = gamerec_version
__date__    = gamerec_date
